/// <reference types="react" />
import * as React from "react";
import { StateEngine } from "./stateEngine";
import { FieldProps, FieldRecord } from "./Field";
import { FieldResult, Validator, InvalidFieldResult } from "./validation/index";
export declare type FieldValue = string | number;
export declare type FieldValueMap = Record<string, FieldValue>;
export declare type FieldMap = Record<string, FieldRecord>;
export declare type FormEventHandler = (values: FieldMap) => void;
export declare type MaybeFormEventHandler = FormEventHandler | undefined;
/**
 * Props supplied to the render component passed to Form
 * TOwnProps is the type of the Form render components own props
 * TFieldOwnProps is the type of the Field render components own props
 */
export interface InjectedFormProps<TOwnProps extends object | void = void, TFieldOwnProps extends object | void = void> {
    Field: React.StatelessComponent<FieldProps<TFieldOwnProps>>;
    form: React.DetailedHTMLProps<React.FormHTMLAttributes<HTMLFormElement>, HTMLFormElement>;
    values: FieldMap;
    meta: {
        valid: boolean;
        submitted: boolean;
        errors: Record<string, InvalidFieldResult>;
        isValidating: boolean;
    };
    actions: {
        reset: () => void;
        submit: () => void;
    };
    ownProps: TOwnProps;
}
/**
 * Props that can be passed to Form
 */
export interface FormProps<T extends object | void = {}, U extends object | void = {}> {
    name: string;
    render: React.SFC<InjectedFormProps<T, U>>;
    validators?: Record<string, Validator<FieldValue>>;
    initialValues?: FieldValueMap;
    onSubmit?: FormEventHandler;
    onSubmitFailed?: FormEventHandler;
    onChange?: FormEventHandler;
    renderProps?: T;
    stateEngine?: StateEngine<FormState>;
}
export interface FormState {
    fields: FieldMap;
    submitted: boolean;
    meta: {
        validation: FieldResult;
    };
}
export declare class Form<T extends object | void, U extends object | void> extends React.Component<FormProps<T, U>, FormState> {
    static defaultProps: Partial<FormProps<any, any>>;
    private Field;
    private stateEngine;
    /**
     * When a Form is instantiated, generate a state engine with initial empty
     * state - this will be filled in by Fields, and create the Field component
     * that will be injected.
     */
    constructor(props: FormProps<T, U>);
    componentWillReceiveProps(nextProps: FormProps<T, U>): boolean;
    /**
     * Generates the Field that will be passed in InjectedFormProps
     */
    private makeField;
    /**
     * Reset everything to initial values
     * State will be fleshed out by Field children when they are recreated
     */
    private reset;
    /**
     * Allows Fields to get to their initial state
     */
    private getInitialValue;
    /**
     * Is every field passing validation
     */
    private allValid;
    /**
     * If there is a validator for field with name of {name}
     * then run it, otherwise return valid
     */
    private validate;
    /**
     * Called by Fields when their value changes
     * If a form onChange handler was passed as a prop, call it
     */
    private handleFieldChange;
    /**
     * On submit call either props.onSubmit or props.onFailedSubmit
     * depending on current validation status
     */
    private handleSubmit;
    render(): React.ReactElement<any> | null;
}
